// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// trace_api.h - (main) description of HL API tracing operations

/*
 * Copyright (c) 2001 Will Day <willday@hpgx.net>
 *
 *    This file is part of Metamod.
 *
 *    Metamod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    Metamod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Metamod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Copyright (c) 2004 OLO
 * This file is a part of PLBot application.
 * You are not allowed to remove these copyrights.
*/



#ifndef PLBOT_API_H
#define PLBOT_API_H

#include <time.h>    // time()
#include <enginecallback.h>  // ALERT()
#include <sdk_util.h>   // UTIL_VarArgs()
#include <api_info.h>

#ifndef __linux__
typedef void (FAR *LINK_ENTITY_FUNC)(entvars_t *);
#else

#include <dlfcn.h>
#define GetProcAddress dlsym

typedef void (*LINK_ENTITY_FUNC)(entvars_t *);
#endif

///#define MY_DEBUGE_MODE

extern cvar_t *plbot_version;
extern cvar_t *plb_control;
extern cvar_t *plb_group;
extern cvar_t *plb_skill;
extern cvar_t *plb_password;
extern cvar_t *plb_config;
extern cvar_t *plb_wptdir;
extern cvar_t *plb_pause;
extern cvar_t *plb_canchat;
extern cvar_t *plb_onlywpt;
extern cvar_t *plb_autoadd;
extern cvar_t *plb_number;
extern cvar_t *plb_jointime;

#define CS_WEAPON_P228           1
#define CS_WEAPON_UNKNOWN2       2
#define CS_WEAPON_SCOUT          3
#define CS_WEAPON_HEGRENADE      4
#define CS_WEAPON_XM1014         5
#define CS_WEAPON_C4             6
#define CS_WEAPON_MAC10          7
#define CS_WEAPON_AUG            8
#define CS_WEAPON_SMOKEGRENADE   9
#define CS_WEAPON_ELITE         10
#define CS_WEAPON_FIVESEVEN     11
#define CS_WEAPON_UMP45         12
#define CS_WEAPON_SG550         13
#define CS_WEAPON_GALIL         14
#define CS_WEAPON_FAMAS         15
#define CS_WEAPON_USP           16
#define CS_WEAPON_GLOCK18       17
#define CS_WEAPON_AWP           18
#define CS_WEAPON_MP5NAVY       19
#define CS_WEAPON_M249          20
#define CS_WEAPON_M3            21
#define CS_WEAPON_M4A1          22
#define CS_WEAPON_TMP           23
#define CS_WEAPON_G3SG1         24
#define CS_WEAPON_FLASHBANG     25
#define CS_WEAPON_DEAGLE        26
#define CS_WEAPON_SG552         27
#define CS_WEAPON_AK47          28
#define CS_WEAPON_KNIFE         29
#define CS_WEAPON_P90           30

#define CS_WEAPON_VEST          40
#define CS_WEAPON_HELMET        41
#define CS_WEAPON_DEFKIT        42
#define CS_WEAPON_GOGGLES       43
#define CS_WEAPON_AMMO          44

typedef struct
{
   char szClassname[64];
   int  iAmmo;     // ammo index for primary ammo
   int  iAmmoMax;  // max primary ammo
   int  iSlot;      // HUD slot (0 based)
   int  iPosition;  // slot position
   int  iId;        // weapon ID
   int  iFlags;     // flags???
   int  iClip;
}
bot_weapon_t;

void FakeClientCommand(edict_t *pBot, const char *arg1, const char *arg2, const char *arg3);

#define MAX_BOT_NAMES  128
#define MAX_BOT_CHAT   512
#define BOT_NAME_LEN   32
#define MAX_CHAT_LEN   80

#define MAX_PARM       48

#define MAX_LAST_NODES 16

#define KLAPS_WAVE "player/pl_pain6.wav"

//Angles in degress for Navigation
#define DEG_5     0.0872
#define DEG_15    0.2617
#define DEG_30    0.5235
#define DEG_45    0.7853
#define DEG_60    1.0471
#define DEG_90    1.5707
#define DEG_270   4.7123

//Parameters
#define maxway parameters[0]
#define bsenss parameters[1]
#define bsensf parameters[2]
#define blevel parameters[3]
#define bheight parameters[4]
#define bunstuck parameters[5]
#define minway parameters[6]
#define bangle parameters[7]
#define showway parameters[8]
#define bprio parameters[9]
#define blnodes parameters[10]
#define bsee parameters[11]
#define btrace parameters[12]

#define BOT_NOACTION       0
#define BOT_DUCKING        (1<<0)
#define BOT_JUMPOFTEN      (1<<1)
#define BOT_STOPED         (1<<2)
#define BOT_JUMPING        (1<<7)
#define player_tHROW_HG    (1<<4)
#define player_tHROW_SG    (1<<5)
#define player_tHROW_FG    (1<<6)

//#define FL_PLBOT_CLIENT (1<<40)

#define LADDER_NONE  0
#define LADDER_DOWN  1
#define LADDER_UP    2

#define RESPAWN_RESPAWN          0
#define RESPAWN_IDLE             1
#define RESPAWN_NEED_TO_RESPAWN  2
#define RESPAWN_IS_RESPAWNING    3
#define RESPAWN_END_ROUND        4
#define RESPAWN_START_ROUND      5
#define RESPAWN_INITIALIZE       6

// game start messages for CS...
#define MSG_CS_IDLE             1
#define MSG_CS_TEAM_SELECT      2
#define MSG_CS_CT_SELECT        3
#define MSG_CS_T_SELECT         4
#define MSG_CS_WEAPON_SELECT    5
#define MSG_CS_RADIO1_SELECT    6
#define MSG_CS_RADIO2_SELECT    7
#define MSG_CS_RADIO3_SELECT    8
#define MSG_CS_EQUIP_SELECT     9

// For Botmenu
#define MSG_CS_BOT_MENU    10
#define MSG_CS_NODE_MENU   11
#define MSG_CS_SKILL_MENU  12
#define MSG_CS_SLAP_MENU   13
#define MSG_CS_KICK_MENU   14
#define MSG_CS_SET_MENU    15
#define MSG_CS_CMD_MENU    16

//Flags for radio commands
#define RADIO_NOMSG           0
#define RADIO_AFFIRMATIVE     (1<<0)
#define RADIO_COVERME         (1<<1)
#define RADIO_FOLLOWME        (1<<2)
#define RADIO_ONPOSITION      (1<<3)
#define RADIO_ENEMYDOWN       (1<<4)
#define RADIO_ENEMYSPOTTED    (1<<5)
#define RADIO_TAKINGFIRE      (1<<6)

//Flags for radio commands
#define USER_NOACTION  0
#define USER_FOLLOWME  (1<<0)
#define USER_COVERME   (1<<1)
#define USER_STICK     (1<<2)
#define USER_FALLBACK  (1<<3)
#define USER_GOGOGO    (1<<4)
#define USER_HOLD      (1<<5)

//Type of map
#define CS_MAP 1
#define DE_MAP 2
#define AS_MAP 3
#define ES_MAP 4

//Who you are
#define CLIENT_UNKNOW     0
#define CLIENT_TERRORIST  (1<<1)
#define CLIENT_CT         (1<<2)
#define CLIENT_VIP        (1<<3)
#define CLIENT_ALLTEAMS   (1<<4)

#define MAX_CVAR   64
#define MAX_CMD    64

class names_s
{
public:
   char* name;
   names_s()
   {
      name = NULL;
   };

   ~names_s()
   {
      free(name);
   };
};

class chat_s
{
public:
   char* text;
   chat_s()
   {
      text = NULL;
   };

   ~chat_s()
   {
      free(text);
   };
};

class cm_cvar_t
{
public:
   char* name;
   float step;
   float max;

   cm_cvar_t()
   {
      name = NULL;
      step = 0.0;
      max = 0.0;
   };

   ~cm_cvar_t()
   {
      free(name);
   };

};

class cmd_t
{
public:
   char* cmd;
   char* cmd_d;

   cmd_t()
   {
      cmd = NULL;
      cmd_d = NULL;
   };

   ~cmd_t()
   {
      free(cmd_d);
      free(cmd);
   };
};

typedef struct
{
   edict_t *pEdict; //Entity of hostage
   edict_t *pPlayer; //Who carries this hostage?
   Vector prev_origin; //Previous origin
   float prev_time; //Check time position
   bool can_use; //Do we can use our hostage?
}
hostage_struct;

typedef struct
{
   Vector place; //Long time ago there was smthing more then only place....
   edict_t *pEdict;
}
bombzone_struct;

typedef struct
{
   Vector place;
   edict_t *pEdict;
}
rescuezone_struct;

typedef struct
{
   Vector place;
   edict_t *pEdict;
}
escapezone_struct;

typedef struct
{
   Vector place;
   edict_t *pEdict;
}
vipescape_struct;

typedef struct
{
   Vector place;
   edict_t *pEdict;
}
rushpoint_struct;

typedef struct
{
   int  iId;     // weapon ID
   int  iClip;   // amount of ammo in the clip
   int  iAmmo;  // amount of ammo in primary reserve
}
bot_current_weapon_t;

typedef struct player_t
{
   bool can_listen; //Do this player can listen radio command (for bot if only one is specified)

   int curr_weapon;
   int botcontrol; //How many bots do we carre

   Vector trEnemy; //Nearest enemy position (even not visible)
   Vector v_prev_origin; //Last origin (for checking if stucked)
   Vector last_enemy_point; //Place where enemy was last visible...
   Vector avoid_angle; //Direction of bots movement (it tries to avoid all walls and cliffs)
   Vector last_enemy_pos;
   Vector last_your_pos;
   Vector throw_direction; //Direction of throwing grenade
   Vector vecHoldPosition; //"Hold this position" place
   bool not_started;

   int flags;

   int bot_skill;
   int start_action; //Used for menu state

   // TheFatal - START
   int msecnum;
   float msecdel;
   float msecval;
   // TheFatal - END

   // things from pev in CBasePlayer...
   int player_team;
   int bot_class;
   int bot_money;

   float blinded_time; //How long do we have to be blinded....
   float prev_time; //Time to check moved distance
   float f_move_speed; //Bot speed

   float moved_distance; //How long we have moved...

   float bot_see_dead_time; //How long we see dead enemy
   float bot_see_enemy_time; //If we see enemy it goes 1 (used for reloading)
   float bot_notice_enemy_time; //How long ago we have noticed enemy (for aming)

   float bot_killchat; //Time to say chat text
   float bot_deadchat;
   float bot_chat;

   float ideal_body; //Degress for body in Y
   float ideal_bodyx; //Degress for body in X

   bot_current_weapon_t current_weapon;  // one current weapon for each bot

   int m_rgAmmo[MAX_AMMO_SLOTS];  // total ammo amounts (1 array for each bot)

   float user_gogogo_time; //How long we should run

   float avoid_time; //How long we should avoid walls (currently not used)

   float checkground_time; //When we should check ground for weapon again...
   float zoom_fov; //Bot zoom..
   float say_time; //Execute Radio time...
   float action_time; //How long we should use action (ducking, jumping)
   float check_action_time; //Next checkig for action (ducking, crouching)
   float shoot_time; //Delay for shooting (depends on skill)

   float check_stuck_time; //For unstuck

   float buy_time; //How fast we can buy next item...

   float weap_change_time; //Blocks fast switching weapon

   int bot_action; //Ducking, crouching and others activites (as flags)
   int hostage_use; //How many hostages we carrie
   int curr_hostage; //Index (for hostage array) of hostage
   int is_shooting; //For emulating pressing button (on/off/on/off
   int bot_spray_logo; //Random index of logo to spray
   int visible_alies_number; //How many friends we can see?
   int visible_enemy_number; //How many enemies we can see?
   int buy_state; //Some kind of turs for buying
   int your_choosed_weapon; //Save ID of weapon which you choosed to pickup from gorund
   int say_radio; //Flag of radio messgae (FOLLOW ME etc.)
   int user_action; //Our user commands (FOLLOW ME, HOLD THIS POSITION etc.)
   int ladder_dir; //In which direction we are going on ladder

   bool can_drop_weapon; //Is it the same kind of weapon (your and on ground) to drop?
   bool inbuyzone; //We are in buyzone (actually not nessesary)
   bool inbombzone; //Are we in bomb zone? (for T)
   bool havebomb; //You have got the bomb!

   float look_around_time; //Frequency of look around updates

   float check_grenade; //How often do we check for greandes
   float update_enemy_time; //For DMC frequency of searching for enemy
   float rush_time; //Time in which bots use only knife to get faster from one place to another

   char fight_name[32];

   double groundAngle;

   edict_t *planedGoal; //As goal in navigation
   edict_t *lastNodes[MAX_LAST_NODES];
   edict_t *currentNodes; //As goal in navigation
   edict_t *pEdict; //Edict of bot
   edict_t *pBotEnemy; //You enemy
   edict_t *pHelpme; //User who called bot by radio messages
   edict_t *pGroundWeapon; //The edict of weapon which we have found on ground and want pickup

   int menu_keys;
   int menu_state;
   int slap_power;
   int players_num;
   edict_t *pList[33];

   bool have_access;
   int who_have_join;
}
player_tt;

inline BOOL IsValidPlayer(edict_t *pEdict)
{
   return (pEdict && STRING(pEdict->v.netname)[0]);
}

inline BOOL IsValidPlayer(player_t *pPlayer)
{
   return (pPlayer && pPlayer->pEdict && STRING(pPlayer->pEdict->v.netname)[0]);
}

#define Generate(a) (RANDOM_LONG(0,a))
#define Chance(a) (RANDOM_LONG(0,a)==a)
#define CPUusage(a) Chance(a)

extern int chat_num;

edict_t *UTIL_FindEntityInSphere( edict_t *pentStart, const Vector &vecCenter, float flRadius );
edict_t *UTIL_FindEntityByString( edict_t *pentStart, const char *szKeyword, const char *szValue );
edict_t *UTIL_FindEntityByClassname( edict_t *pentStart, const char *szName );
edict_t *UTIL_FindEntityByModelname( edict_t *pentStart, const char *szName );

void UTIL_SelectWeapon(edict_t *pEdict, int weapon_index);

int UTIL_GetMenuIndex(edict_t *pEntity);

player_t *UTIL_GetPlayerPointer(edict_t *pEdict);

inline BOOL IsAlive(edict_t *pEdict)
{
   return (pEdict && (pEdict->v.deadflag == DEAD_NO) && (pEdict->v.health > 0) && (pEdict->v.movetype != MOVETYPE_NOCLIP));
}

bool FInViewCone(Vector *pOrigin, edict_t *pEdict, float deg);
bool FInViewConeOfRadar(Vector *pOrigin, edict_t *pEdict, float deg);
bool FVisible( const Vector &vecOrigin, edict_t *pEdict );

Vector Center(edict_t *pEdict);

void UTIL_ShowMenu( edict_t *pEdict, int slots, int displaytime, bool needmore, char *pText );
void UTIL_BuildFileName(char *filename, char *arg1, char *arg2);

extern int Maptype;
extern bool RoundGoing;
extern int parameters[MAX_PARM];
extern int developme;
extern player_t bots[33];

#ifdef MY_DEBUGE_MODE
extern edict_t *listenserver_edict;
#endif

extern float angle_cos_30[7];
extern float angle_sin_30[7];

extern player_t bots[33];

extern bool bombplanted;
extern bool bombdroped;
extern bool bombnotfound;
extern edict_t *bombplaced;

extern int Maptype;
extern int t_group_number;
extern int ct_group_number;

extern float bomb_check_time;

extern int rushpoint_num;
extern int nodes_on;
extern int parameters[MAX_PARM];

extern globalvars_t  *gpGlobals;
extern player_t bots[33];


extern cm_cvar_t cvar_game[MAX_CVAR];
extern int cvars_num;
extern cmd_t cstcmd_names[MAX_CMD];
extern int cstcmd_num;

extern bot_weapon_t weapon_defs[MAX_WEAPONS];
extern player_t bots[33];
extern float bomb_check_time;
extern bool bombdroped;

extern int parameters[MAX_PARM];
extern int killchat_count;
extern int chat_count;
extern int deadchat_count;
extern int number_names;
extern int botchat;

extern bool need_to_open_cfg;
extern bool spawn_time_reset;

extern int parameters[MAX_PARM];
extern int Maptype;

extern bool not_found_goal;
extern bool bombplanted;
extern bool bombdroped;
extern bool bombnotfound;

extern edict_t *bombplaced;
extern player_t bots[33];

extern int Maptype;
extern int developme;

extern bool RoundGoing;
extern bool bombplanted;

extern int parameters[MAX_PARM];

extern bool bombplanted;
extern bool bombdroped;
extern bool bombnotfound;

extern names_s bot_names[MAX_BOT_NAMES];

extern chat_s chat_text[MAX_BOT_CHAT];

extern rushpoint_struct rushpoint[12];
extern int rushpoint_num;

extern player_t bots[33];
extern int m_spriteTexture;
extern char g_argv[1024];

extern int parameters[MAX_PARM];
extern bot_weapon_t weapon_defs[MAX_WEAPONS];

extern bool RoundGoing;

extern int Maptype;
extern int parameters[MAX_PARM];
extern int gmsgShowMenu;
extern int vkeys;

extern Vector NodesToLoad[256];
extern player_t bots[33];

extern player_t bots[33];
extern int nodes_on;

extern cm_cvar_t cvar_game[MAX_CVAR];
extern int cvars_num;

extern cmd_t cstcmd_names[MAX_CMD];
extern int cstcmd_num;


void BotSpawnInit( player_t *pBot, int spawn_state );
void BotCreate( edict_t *pPlayer, const char *arg1, const char *arg2);
void BotShootAtEnemy( player_t *pBot );
void BotThink( player_t *pBot );
void BotUseAction( player_t *pBot);
void BotUnstuck(player_t *pBot, float moved_distance);
void BotFindAction(player_t *pBot);
void ClientKill( edict_t *pEntity );
void ClientCommand( edict_t *pEntity );
void RestartRound(int bot_index);
void Botmenu(player_t *pEntity, int iMenu);
void FreeMyHostage(edict_t *pEntity);
void FindBomb();
void CheckHostages();
void BuildCfgNameFile();
void BotFindAction(player_t *pBot, float EnemyDist, edict_t *pEnemy);
void LookAround(player_t *pBot);
void FreeHostage(int i);
void ModThink(player_t *pPlayer);
void NullLastGoals(player_t *pBot);
void FindGoalTarget(player_t *pBot);

#ifdef MY_DEBUGE_MODE
void DrawBeam(edict_t *pEntity, Vector start, Vector end, int red, int green, int blue);
#endif

bool FromYourTeam(player_t *pBot, player_t *pObject);
bool HaveAction(player_t *pBot);
bool IsShooting(edict_t *pEdict);
bool HaveRifle(player_t *pBot);
bool HaveSniperRifle(player_t *pBot);

inline BOOL HaveGrenadeInHand(player_t *pBot)
{
   return (pBot->current_weapon.iId==CS_WEAPON_HEGRENADE||
           pBot->current_weapon.iId==CS_WEAPON_SMOKEGRENADE||
           pBot->current_weapon.iId==CS_WEAPON_FLASHBANG);
}

bool BotFireWeapon(float distance, player_t *pBot, int weapon_choice);
bool IsOnLadder( player_t *pBot );
bool OnBombZone(player_t *pBot);
bool OnThisSameLevel(Vector v_src, Vector v_dest, int high);
bool OnThisSameArea(Vector v_src, Vector v_dest, int area);
bool BotBuyEquip(player_t *pBot );
bool IsAbleToHit (player_t *pBot);
bool HaveToUseGrenade(player_t *pBot);
bool EnemyIsVisible( player_t *pBot, Vector vecDis);
bool CheckForSmokeGrenade(player_t *pBot, float distance);
bool CheckIfCanJump(player_t *pBot, IGNORE_MONSTERS igmon);
bool CheckIfCanJumpWhole(player_t *pBot);
//bool FoundLadder(player_t *pBot);
bool FoundBreakable(player_t *pBot);
bool GrabHostage(player_t *pBot);

bool ThrowGrenade(player_t *pBot);
bool SwitchToWeapon(player_t *pBot);
bool DecideToThrowGrenade(player_t *pBot);
bool HaveOneZoom(player_t *pBot);
bool HaveTwoZoom(player_t *pBot);
bool AbleToSwitchLaser(edict_t *pPlayer);
bool CheckIfCanDuck(player_t *pBot);
bool BotSprayLogo( player_t *pBot );
bool NavigateCustomDir(player_t *pBot, Vector vecSrc, Vector vecDest, Vector &vecAvoid,  float dist, float sens);

int InFieldOfView( player_t *pBot, Vector dest, float view_angle );
int CheckTheAngle(Vector src, Vector dest);
int FindHostage(player_t *pBot);

float BotChangeAngle(float &angle_current, float ideal_angle, float speed, bool body );

inline BOOL IsThrowingGrenade(edict_t *pEdict)
{
   //These animation sequences are for throwing greande
   return ((pEdict->v.sequence == 57) || (pEdict->v.sequence == 59));
}

Vector NavigateMe(player_t *pBot);
Vector BotPointAt(player_t *pBot , Vector v_target, bool visbility);
Vector FixAngle(edict_t *pEdict, Vector angle);
Vector BotBodyTarget( edict_t *pBotEnemy, player_t *pBot );

bool FindBestWeaponOnGround(player_t *pBot, Vector &v_avoid_dir, float sens);
bool BotGetHostages(player_t *pBot, Vector &vecAvoid, float sens);
bool BotGetToC4(player_t *pBot, Vector &vecAvoid, float sens);
bool BotGetToBombZone(player_t *pBot, Vector &vecAvoid, float sens);
void FindRushPointNodes();

edict_t *BotFindEnemy( player_t *pBot );
edict_t *Finditem( player_t *pBot , Vector vecSrc, char item_flag[40], int who, int how_near, bool classname);

inline BOOL BotNoAction(player_t *pBot)
{
   //Check if bot have action (action time should be greater than gpGlobals time)
   return ((pBot->bot_action == BOT_NOACTION) && (pBot->action_time < gpGlobals->time));
}

//extern inline player_t *GetPlayerPointer(int a);
//extern inline player_t *GetPlayerPointer(edict_t *pEntity);
bool OnThisSameArea(Vector v_src, Vector v_dest, int area);
bool OnThisSameLevel(Vector v_src, Vector v_dest, int high);

char* UTIL_VarArgs( char *format, ... );

void MenuSettingsList(player_t *pPlayer, int state);
void MenuCmdList(player_t *pPlayer, int state);
void MenuCmdList(player_t *pPlayer, int state);

void MenuPlayersList(player_t *pPlayer, const char *menu_header, int pnum, int itype, int state);

void BotClient_VGUIMenu(void *p, player_t *pBot);
void BotClient_WeaponList(void *p, player_t *pBot);
void BotClient_CurrentWeapon(void *p, player_t *pBot);
void BotClient_AmmoX(void *p, player_t *pBot);
void BotClient_AmmoPickup(void *p, player_t *pBot);
void BotClient_WeaponPickup(void *p, player_t *pBot);
void BotClient_Damage(void *p, player_t *pBot);
void BotClient_Money(void *p, player_t *pBot);
void BotClient_DeathMsg(void *p, player_t *pBot);
void BotClient_ScreenFade(void *p, player_t *pBot);
void BotClient_TextMsg(void *p, player_t *pBot);
void BotClient_SendAudio(void *p, player_t *pBot);
void BotClient_Icon(void *p, player_t *pBot);
void BotClient_SetFOV(void *p, player_t *pBot);
void BotClient_HLTV(void *p, player_t *pBot);

void PlayerInit(edict_t *pEdict);
void FindObjectives();
bool ApplyRadioCommand(player_t *pBot);

inline BOOL FStrEqI(const char*sz1, const char*sz2)
{
   return (strcmpi(sz1, sz2) == 0);
}
inline BOOL FStrnEq(const char*sz1, const char*sz2, int num)
{
   return (strncmp(sz1, sz2, num) == 0);
}

void plbot_init(void);

#define CREATE_FAKE_CLIENT (*g_engfuncs.pfnCreateFakeClient)
#define RUNPLAYERMOVE  (*g_engfuncs.pfnRunPlayerMove)
#define CLIENT_PRINT   (*g_engfuncs.pfnClientPrintf)
#define ANGLEVECTORS   (*g_engfuncs.pfnAngleVectors)
#define CVAR_DIRSET_STRING (*g_engfuncs.pfnCvar_DirectSet)


#define GET_IPLAYERPOINTER(a) (&bots[a])
#define GET_EPLAYERPOINTER(a) (&bots[ENTINDEX(a)])

#define FromYourTeam(a,b) ((a)->player_team & (b)->player_team) //Yes he is from our team

#define GetGunPosition(pEdict) ((pEdict)->v.origin + (pEdict)->v.view_ofs)

#define VECBMODELORIGIN(pEdict) ((pEdict)->v.absmin + ((pEdict)->v.size * 0.5))

#endif /* TRACE_API_H */
